#ifndef MAPPER_H
#define MAPPER_H

#include "SolverMatrix.h"
#include <vector>
namespace AMDiS {
/*  struct ScalarMapper {
    ScalarMapper(int nrow, int ncol):
      nrow(nrow), ncol(ncol) {}
    inline int row(int r) { return r; }
    inline int col(int c) { return c; }
    inline int nRow() { return nrow; }
    inline int nCol() { return ncol; }
  protected:
    int nrow;
    int ncol;
  };
*/
  struct VectorialMapper 
  {

    VectorialMapper(const SolverMatrix<Matrix<DOFMatrix* > >& sm ) :
      nComp(sm.getOriginalMat()->getSize()), 
      rowOffset(0), colOffset(0), nrow(0), ncol(0), sizes(nComp)
    {
      const Matrix<DOFMatrix* >& orMat(*sm.getOriginalMat());
      const int ns = orMat.getSize();
      for (int i= 0; i < ns; i++) {
	sizes[i] = orMat[i][i]->getFeSpace()->getAdmin()->getUsedSize();
	nrow += sizes[i];
      }
      ncol = nrow;
    }

    VectorialMapper(unsigned int nComp, int nDOFperComp) :
      nComp(nComp), rowOffset(0), colOffset(0), 
      nrow(nComp*nDOFperComp), ncol(nrow),sizes(nComp)
    {
       for (unsigned int i(0); i < nComp; ++i)
         sizes[i] = nDOFperComp;
    }
      
    inline void setRow( unsigned int r) { 
	    assert( r <= sizes.size() );
	    rowOffset = sum( r); 
    }

    inline void setCol( unsigned int c) { colOffset = sum( c); }

    inline int row(int r) const { return r + rowOffset; }

    inline int col(int c) const { return c + colOffset; }

    inline int nCol() const { return ncol; }

    inline int nRow() const { return nrow; }

  protected:

    ///compute the sum of sizes from [0, end)
    inline unsigned int sum(unsigned int end) const 
    {
      unsigned int ret(0);
      for (unsigned int i(0); i < end; ++i)
	ret += sizes[i];
      return ret;
    }

    int nComp;
    int rowOffset;
    int colOffset;
    int nrow;
    int ncol;
    std::vector< unsigned int > sizes;
  };
}

#endif
